from .barometer_utils import process_barometer_data
from .data_loader import load_sensor_data_from_log
from .data_preprocessor import preprocess_data
from .floor_map import FloorMap
from .visualizer import plot_trajectory

__all__ = [
    "FloorMap",
    "preprocess_data",
    "plot_trajectory",
    "process_barometer_data",
    "load_sensor_data_from_log",
]



from .column_name import (
    ACC_X,
    ACC_Y,
    ACC_Z,
    ANGLE,
    BLE_ADDRESS,
    COORDINATE_X,
    COORDINATE_Y,
    FLOOR_NAME,
    GYRO_X,
    GYRO_Y,
    GYRO_Z,
    POS_X,
    POS_Y,
    POS_Z,
    PRESSURE,
    QUATERNION_0,
    QUATERNION_1,
    QUATERNION_2,
    QUATERNION_3,
    RSSI,
    STEP_LENGTH,
    TIMESTAMP,
)
from .parameter import DEFAULT_STEP_LENGTH
from .path import FLOOR_MAP_PATH, LOG_FILE_PATH

__all__ = [
    "ACC_X",
    "ACC_Y",
    "ACC_Z",
    "BLE_ADDRESS",
    "FLOOR_NAME",
    "GYRO_X",
    "GYRO_Y",
    "GYRO_Z",
    "POS_X",
    "POS_Y",
    "POS_Z",
    "PRESSURE",
    "QUATERNION_0",
    "QUATERNION_1",
    "QUATERNION_2",
    "QUATERNION_3",
    "RSSI",
    "TIMESTAMP",
    "COORDINATE_X",
    "COORDINATE_Y",
    "ANGLE",
    "FLOOR_MAP_PATH",
    "LOG_FILE_PATH",
    "STEP_LENGTH",
    "DEFAULT_STEP_LENGTH",
]



import numpy as np
from PIL import Image


class FloorMap:
    def __init__(
        self,
        floor_name: str,
        floor_map_path: str,
        dx: float,
        dy: float,
    ) -> None:
        self.floor_name = floor_name
        self.floor_map_data = FloorMap.__load_floor_map(floor_map_path)
        # ピクセル間の距離
        self.dx = dx
        self.dy = dy

    @staticmethod
    def __load_floor_map(
        floor_map_path: str,
    ) -> np.ndarray:
        map_image_path = floor_map_path
        map_image = Image.open(map_image_path)

        return np.array(map_image, dtype=bool)



import numpy as np
import pandas as pd


def process_barometer_data(
    baro_data: pd.DataFrame,
    window_size: int = 50,
    pressure_threshold: float = 0.5,
) -> pd.DataFrame:
    """Process barometer data to identify stable periods and estimate relative altitude changes.

    Args:
    ----
        baro_data (pd.DataFrame): Raw barometer data.
        window_size (int): Size of the rolling window for smoothing.
        pressure_threshold (float): Threshold for identifying stable pressure periods.

    Returns:
    -------
        pd.DataFrame: Processed barometer data with altitude estimates.

    """
    # Smooth pressure data
    baro_data["smooth_pressure"] = (
        baro_data["pressure"].rolling(window=window_size, center=True).mean()
    )

    # Identify stable periods
    baro_data["pressure_diff"] = baro_data["smooth_pressure"].diff().abs()
    baro_data["is_stable"] = baro_data["pressure_diff"] < pressure_threshold

    # Estimate relative altitude changes
    reference_pressure = baro_data["smooth_pressure"].iloc[0]
    baro_data["relative_altitude"] = 44330 * (
        1 - (baro_data["smooth_pressure"] / reference_pressure) ** (1 / 5.255)
    )

    return baro_data


def detect_floor_changes(
    baro_data: pd.DataFrame,
    altitude_threshold: float = 2.5,
) -> pd.DataFrame:
    """Detect potential floor changes based on barometer data.

    Args:
    ----
        baro_data (pd.DataFrame): Processed barometer data.
        altitude_threshold (float): Threshold for detecting a floor change.

    Returns:
    -------
        pd.DataFrame: Data frame with detected floor changes.

    """
    floor_changes = []
    current_floor = 0

    for i in range(1, len(baro_data)):
        altitude_change = (
            baro_data["relative_altitude"].iloc[i]
            - baro_data["relative_altitude"].iloc[i - 1]
        )
        if abs(altitude_change) > altitude_threshold:
            current_floor += np.sign(altitude_change)
            floor_changes.append(
                {
                    "timestamp": baro_data["timestamp"].iloc[i],
                    "floor": int(current_floor),
                    "altitude_change": altitude_change,
                },
            )

    return pd.DataFrame(floor_changes)



from __future__ import annotations

import numpy as np
import pandas as pd
from scipy import signal

from src.palkia.const import TIMESTAMP


def preprocess_data(
    data: pd.DataFrame,
    lowpass_freq: float = 5.0,
    sampling_rate: float = 100.0,
) -> pd.DataFrame:
    """Preprocess sensor data by applying a low-pass filter and resampling.

    Args:
    ----
        data (pd.DataFrame): Input sensor data.
        lowpass_freq (float): Cutoff frequency for the low-pass filter.
        sampling_rate (float): Desired sampling rate after resampling.

    Returns:
    -------
        pd.DataFrame: Preprocessed sensor data.

    """
    # Apply low-pass filter
    nyquist_freq = 0.5 * sampling_rate
    normal_cutoff = lowpass_freq / nyquist_freq
    b, a = signal.butter(4, normal_cutoff, btype="low", analog=False)

    filtered_data = pd.DataFrame()
    for column in data.columns:
        if column != TIMESTAMP:
            filtered_data[column] = signal.filtfilt(b, a, data[column])

    filtered_data[TIMESTAMP] = data[TIMESTAMP]

    # Resample data
    return (
        filtered_data.set_index(TIMESTAMP)
        .resample(f"{1/sampling_rate}S")
        .mean()
        .reset_index()
    )


def align_sensor_data(
    acc_data: pd.DataFrame,
    gyro_data: pd.DataFrame,
    baro_data: pd.DataFrame,
) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Align different sensor data based on their timestamps.

    Args:
    ----
        acc_data (pd.DataFrame): Accelerometer data.
        gyro_data (pd.DataFrame): Gyroscope data.
        baro_data (pd.DataFrame): Barometer data.

    Returns:
    -------
        Tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]: Aligned sensor data.

    """
    # Implement time alignment logic here
    # This could involve interpolation or resampling to a common time base
    # For simplicity, let's assume we resample all data to the accelerometer's timestamps

    common_time = acc_data[TIMESTAMP]

    aligned_gyro = (
        gyro_data.set_index(TIMESTAMP)
        .reindex(common_time, method="nearest")
        .reset_index()
    )
    aligned_baro = (
        baro_data.set_index(TIMESTAMP)
        .reindex(common_time, method="nearest")
        .reset_index()
    )

    return acc_data, aligned_gyro, aligned_baro


def match_data(
    data_df: pd.DataFrame,
    reference_times: pd.Series,
    time_column: str = TIMESTAMP,
    tolerance: float = 0.005,
) -> pd.DataFrame:
    """時系列データから指定された時間に最も近いデータポイントを抽出します。.

    Args:
    ----
        data_df (pd.DataFrame): 元のデータフレーム
        reference_times (pd.Series): 抽出したい参照時間のシリーズ
        time_column (str): データフレーム内の時間列の名前
        tolerance (float): 時間の一致を判断する際の許容誤差 (秒)

    Returns:
    -------
        pd.DataFrame: 抽出されたデータポイントを含むデータフレーム

    """
    # マッチした行を格納するためのリストを初期化
    matched_rows = []

    # 各参照時間に対してループ
    for t in reference_times:
        # データフレームから、指定された時間に近いデータポイントを抽出
        # np.isclose() を使用して、指定された許容誤差内のデータを選択
        matched_row = data_df[np.isclose(data_df[time_column], t, atol=tolerance)]

        # マッチした行が存在する場合のみ、リストに追加
        if not matched_row.empty:
            matched_rows.append(matched_row)

    # マッチした行が1つ以上存在する場合
    if matched_rows:
        # すべてのマッチした行を1つのDataFrameに結合
        # axis=0: 行方向(縦方向)に結合
        # ignore_index=True: 元のインデックスを無視し、新しいインデックスを生成
        return pd.concat(matched_rows, axis=0, ignore_index=True)

    # マッチする行が1つも見つからなかった場合、空のDataFrameを返す
    return pd.DataFrame()



from __future__ import annotations

from typing import TYPE_CHECKING

import matplotlib.pyplot as plt
import numpy as np

if TYPE_CHECKING:
    import pandas as pd

    from src.palkia.utils.floor_map import FloorMap

from src.palkia.const import (
    ACC_X,
    ACC_Y,
    ACC_Z,
    COORDINATE_X,
    COORDINATE_Y,
    GYRO_X,
    GYRO_Y,
    GYRO_Z,
    PRESSURE,
    TIMESTAMP,
)

# Constants for plot configuration
FIGURE_SIZE = (10, 10)
SCATTER_SIZE = 5
START_POINT_COLOR = "#674598"
END_POINT_COLOR = "red"
STRAT_END_POINT_SIZE = 50
GROUND_TRUTH_COLOR = "blue"
GROUND_TRUTH_STYLE = "--"


def plot_trajectory(
    trajectory: pd.DataFrame,
    ground_truth: pd.DataFrame | None = None,
    floor_map: FloorMap | None = None,
    *,
    figsize: tuple[int, int] = FIGURE_SIZE,
    scatter_size: int = SCATTER_SIZE,
    start_point_color: str = START_POINT_COLOR,
    end_point_color: str = END_POINT_COLOR,
    start_end_point_size: int = STRAT_END_POINT_SIZE,
    ground_truth_color: str = GROUND_TRUTH_COLOR,
    ground_truth_style: str = GROUND_TRUTH_STYLE,
) -> None:
    """Plot the estimated trajectory, optionally with ground truth and floor map.

    Args:
    ----
        trajectory (pd.DataFrame): Estimated trajectory data.
        ground_truth (pd.DataFrame, optional): Ground truth trajectory data.
        floor_map (FloorMap, optional): Floor map data.

        figsize (tuple[int, int], optional): Size of the figure.
        scatter_size (int, optional): Size of the scatter points.
        start_point_color (str, optional): Color of the start point.
        end_point_color (str, optional): Color of the end point.
        start_end_point_size (int, optional): Size of the start and end points.
        ground_truth_color (str, optional): Color of the ground truth line.
        ground_truth_style (str, optional): Style of the ground truth line.

    """
    plt.figure(figsize=figsize)

    _plot_estimated_trajectory(trajectory, scatter_size)
    _plot_start_end_points(
        trajectory,
        start_point_color,
        end_point_color,
        start_end_point_size,
    )

    if ground_truth is not None:
        _plot_ground_truth(ground_truth, ground_truth_color, ground_truth_style)

    if floor_map is not None:
        _plot_floor_map(floor_map)

    _set_plot_properties()
    plt.show()


def _plot_estimated_trajectory(
    trajectory: pd.DataFrame,
    scatter_size: int = SCATTER_SIZE,
) -> None:
    """Plot the estimated trajectory with a color gradient based on time."""
    scatter = plt.scatter(
        trajectory[COORDINATE_X],
        trajectory[COORDINATE_Y],
        c=trajectory[TIMESTAMP],
        cmap="rainbow",
        s=scatter_size,
    )
    colorbar = plt.colorbar(scatter)
    colorbar.set_label("time(s)", fontsize=12)


def _plot_start_end_points(
    trajectory: pd.DataFrame,
    start_point_color: str = START_POINT_COLOR,
    end_point_color: str = END_POINT_COLOR,
    start_end_point_size: int = STRAT_END_POINT_SIZE,
) -> None:
    """Plot the start and end points of the trajectory."""
    plt.scatter(
        trajectory[COORDINATE_X].iloc[0],
        trajectory[COORDINATE_Y].iloc[0],
        c=start_point_color,
        s=start_end_point_size,
        label="Start",
    )
    plt.scatter(
        trajectory[COORDINATE_X].iloc[-1],
        trajectory[COORDINATE_Y].iloc[-1],
        c=end_point_color,
        s=start_end_point_size,
        label="End",
    )


def _plot_ground_truth(
    ground_truth: pd.DataFrame,
    color: str = GROUND_TRUTH_COLOR,
    style: str = GROUND_TRUTH_STYLE,
) -> None:
    """Plot the ground truth trajectory."""
    plt.plot(
        ground_truth["x"],
        ground_truth["y"],
        f"{color}{style}",
        label="Ground Truth",
    )


def _plot_floor_map(floor_map: FloorMap) -> None:
    """Plot the floor map as a background."""
    plt.title(floor_map.floor_name)
    plt.imshow(
        np.rot90(floor_map.floor_map_data),
        extent=(
            0,
            floor_map.floor_map_data.shape[0] * floor_map.dx,
            0,
            floor_map.floor_map_data.shape[1] * floor_map.dy,
        ),
        cmap="binary",
        alpha=0.5,
    )


def _set_plot_properties() -> None:
    """Set general properties of the plot."""
    plt.xlabel("X (m)")
    plt.ylabel("Y (m)")
    plt.title("Trajectory")
    plt.legend()
    plt.grid()
    plt.axis("equal")


def plot_sensor_data(
    acc_data: pd.DataFrame,
    gyro_data: pd.DataFrame,
    baro_data: pd.DataFrame,
) -> None:
    """Plot raw sensor data with increased spacing between subplots.

    Args:
    ----
        acc_data (pd.DataFrame): Accelerometer data.
        gyro_data (pd.DataFrame): Gyroscope data.
        baro_data (pd.DataFrame): Barometer data.

    """
    fig, (ax1, ax2, ax3) = plt.subplots(3, 1, figsize=(12, 9))  # 高さを増やす

    # Plot accelerometer data
    ax1.plot(acc_data[TIMESTAMP], acc_data[ACC_X], label="X")
    ax1.plot(acc_data[TIMESTAMP], acc_data[ACC_Y], label="Y")
    ax1.plot(acc_data[TIMESTAMP], acc_data[ACC_Z], label="Z")
    ax1.set_title("Accelerometer Data", fontsize=16)
    ax1.set_xlabel("Time", fontsize=12)
    ax1.set_ylabel("Acceleration (m/s²)", fontsize=12)
    ax1.legend(fontsize=10)
    ax1.tick_params(labelsize=10)

    # Plot gyroscope data
    ax2.plot(gyro_data[TIMESTAMP], gyro_data[GYRO_X], label="X")
    ax2.plot(gyro_data[TIMESTAMP], gyro_data[GYRO_Y], label="Y")
    ax2.plot(gyro_data[TIMESTAMP], gyro_data[GYRO_Z], label="Z")
    ax2.set_title("Gyroscope Data", fontsize=16)
    ax2.set_xlabel("Time", fontsize=12)
    ax2.set_ylabel("Angular Velocity (rad/s)", fontsize=12)
    ax2.legend(fontsize=10)
    ax2.tick_params(labelsize=10)

    # Plot barometer data
    if PRESSURE in baro_data.columns:
        ax3.plot(baro_data[TIMESTAMP], baro_data[PRESSURE])
        ax3.set_title("Barometer Data", fontsize=16)
        ax3.set_xlabel("Time", fontsize=12)
        ax3.set_ylabel("Pressure (hPa)", fontsize=12)
        ax3.tick_params(labelsize=10)
    else:
        ax3.text(
            0.5,
            0.5,
            "No barometer data available",
            ha="center",
            va="center",
            fontsize=14,
        )

    # サブプロット間の間隔を調整
    fig.subplots_adjust(hspace=0.4)  # 垂直方向の間隔を増やす

    plt.show()



# src/const/column_names.py

# 共通
TIMESTAMP = "ts"

# 加速度計
ACC_X = "acc_x"
ACC_Y = "acc_y"
ACC_Z = "acc_z"

# ジャイロスコープ
GYRO_X = "gyro_x"
GYRO_Y = "gyro_y"
GYRO_Z = "gyro_z"

# 角度 ジャイロを積分したもの
ANGLE = "angle_x"

# 気圧計
PRESSURE = "pressure"

# BLE
BLE_ADDRESS = "bdaddress"
RSSI = "rssi"

# 位置
POS_X = "x"
POS_Y = "y"
POS_Z = "z"

# 四元数
QUATERNION_0 = "q0"
QUATERNION_1 = "q1"
QUATERNION_2 = "q2"
QUATERNION_3 = "q3"

# フロア名
FLOOR_NAME = "floor_name"

# 座標
COORDINATE_X = "x"
COORDINATE_Y = "y"


# step
STEP_LENGTH = "step_length"



import numpy as np
import pandas as pd

from .pdr_estimator import PDREstimator


class ThreeDimensionalEstimator:
    def __init__(self, pdr_estimator: PDREstimator) -> None:
        self.pdr_estimator = pdr_estimator

    def estimate_3d_trajectory(
        self,
        acc_data: pd.DataFrame,
        gyro_data: pd.DataFrame,
        baro_data: pd.DataFrame,
    ) -> pd.DataFrame:
        # 2D軌跡の推定
        trajectory_2d = self.pdr_estimator.estimate_trajectory(acc_data, gyro_data)

        # 気圧データから高度を推定
        height = self.__estimate_height_from_pressure(baro_data)

        # 3D軌跡の生成
        trajectory_3d = trajectory_2d.copy()
        trajectory_3d["z"] = np.interp(
            trajectory_3d["timestamp"],
            baro_data["timestamp"],
            height,
        )

        return trajectory_3d

    def __estimate_height_from_pressure(self, baro_data: pd.DataFrame) -> np.ndarray:
        # 気圧から高度への変換 簡易実装
        pressure_sea_level = 1013.25  # hPa
        height = 44330 * (
            1 - (baro_data["pressure"] / pressure_sea_level) ** (1 / 5.255)
        )

        return height.to_numpy()



from __future__ import annotations

from typing import Literal

import numpy as np
import pandas as pd

from src.palkia.const import ANGLE, COORDINATE_X, COORDINATE_Y, STEP_LENGTH, TIMESTAMP


class TrajectoryCalculator:
    def __init__(
        self,
        initial_point: dict[Literal["x", "y"], float] | None = None,
    ) -> None:
        self.initial_point = initial_point or {"x": 0, "y": 0}

    def calculate_trajectory(self, steps_data: pd.DataFrame) -> pd.DataFrame:
        x_moves = steps_data[STEP_LENGTH] * np.cos(steps_data[ANGLE])
        y_moves = steps_data[STEP_LENGTH] * np.sin(steps_data[ANGLE])

        initial_point_df = pd.DataFrame(
            {
                TIMESTAMP: [steps_data[TIMESTAMP][0]],
                COORDINATE_X: [self.initial_point["x"]],
                COORDINATE_Y: [self.initial_point["y"]],
            },
        )

        trajectory = pd.concat(
            [
                initial_point_df,
                pd.DataFrame(
                    {
                        TIMESTAMP: steps_data[TIMESTAMP],
                        COORDINATE_X: self.initial_point["x"] + x_moves.cumsum(),
                        COORDINATE_Y: self.initial_point["y"] + y_moves.cumsum(),
                    },
                ),
            ],
        )

        return trajectory.dropna(how="all")



import numpy as np
import pandas as pd

from src.palkia.const import ANGLE, GYRO_X, TIMESTAMP
from src.palkia.utils.data_preprocessor import match_data


class OrientationEstimator:
    def __init__(self, drift_correction_factor: float = 0) -> None:
        self.drift_correction_factor = drift_correction_factor

    def __calculate_full_orientation(self, gyro_data: pd.DataFrame) -> pd.DataFrame:
        # 角速度を積分して角度を計算
        orientation = pd.DataFrame()
        orientation[TIMESTAMP] = gyro_data[TIMESTAMP]
        orientation[ANGLE] = np.cumsum(
            gyro_data[GYRO_X]
            * np.diff(gyro_data[TIMESTAMP], prepend=gyro_data[TIMESTAMP].iloc[0]),
        )

        # ドリフト補正
        time_elapsed = gyro_data[TIMESTAMP] - gyro_data[TIMESTAMP].iloc[0]
        orientation[ANGLE] -= (
            self.drift_correction_factor * time_elapsed * orientation[ANGLE].iloc[-1]
        )

        return orientation

    def estimate_step_orientations(
        self,
        gyro_data: pd.DataFrame,
        step_times: pd.Series,
    ) -> pd.DataFrame:
        # 全体の方向を計算
        full_orientation = self.__calculate_full_orientation(gyro_data)

        # 歩行ステップ時の方向データを抽出
        return match_data(full_orientation, step_times)



from __future__ import annotations

from typing import Protocol

import numpy as np
import pandas as pd
from scipy.signal import find_peaks

from src.palkia.const import (
    ACC_X,
    ACC_Y,
    ACC_Z,
    DEFAULT_STEP_LENGTH,
    STEP_LENGTH,
    TIMESTAMP,
)


class Predictor(Protocol):
    def predict(self, x: np.ndarray) -> np.ndarray: ...


class StepEstimator:
    def __init__(
        self,
        peak_threshold: float = 12,
        window_size: int = 10,
        step_length: float = DEFAULT_STEP_LENGTH,
        step_length_model: Predictor | None = None,
    ) -> None:
        self.peak_threshold = peak_threshold
        self.window_size = window_size
        self.step_length_model = step_length_model
        self.step_length = step_length

    def estimate_steps(
        self, acc_data: pd.DataFrame, step_orientations: pd.DataFrame
    ) -> pd.DataFrame:
        step_times = self.detect_step_times(acc_data)
        step_lengths = self.__estimate_step_lengths(acc_data, step_orientations)

        return pd.DataFrame({TIMESTAMP: step_times, STEP_LENGTH: step_lengths})

    def detect_step_times(self, acc_data: pd.DataFrame) -> np.ndarray:
        acc_magnitude = np.sqrt(
            acc_data[ACC_X] ** 2 + acc_data[ACC_Y] ** 2 + acc_data[ACC_Z] ** 2,
        )
        acc_smoothed = pd.Series(acc_magnitude).rolling(window=self.window_size).mean()
        peaks, _ = find_peaks(
            acc_smoothed,
            height=self.peak_threshold,
            distance=self.window_size,
        )
        return acc_data.iloc[peaks][TIMESTAMP].to_numpy()

    def __estimate_step_lengths(
        self, acc_data: pd.DataFrame, step_orientations: pd.DataFrame
    ) -> np.ndarray:
        if self.step_length_model is None:
            return np.full(len(step_orientations), self.step_length)

        features = self.__extract_features(acc_data, step_orientations)
        return self.step_length_model.predict(features)

    def __extract_features(
        self, _: pd.DataFrame, step_orientations: pd.DataFrame
    ) -> np.ndarray:
        return np.zeros((len(step_orientations), 10))  # ダミーの特徴量



from __future__ import annotations

from collections import defaultdict
from pathlib import Path

import pandas as pd

from src.palkia.const import (
    ACC_X,
    ACC_Y,
    ACC_Z,
    BLE_ADDRESS,
    FLOOR_NAME,
    GYRO_X,
    GYRO_Y,
    GYRO_Z,
    POS_X,
    POS_Y,
    POS_Z,
    PRESSURE,
    QUATERNION_0,
    QUATERNION_1,
    QUATERNION_2,
    QUATERNION_3,
    RSSI,
    TIMESTAMP,
)


def read_log_data(log_file_path: str) -> dict[str, pd.DataFrame]:
    data: dict[str, list[dict[str, float | int | str]]] = defaultdict(list)

    with Path(log_file_path).open() as fp:
        for line in fp:
            line_contents = line.rstrip("\n").split(";")
            data_type = line_contents[0]
            if data_type == "BLUE":
                data["BLUE"].append(
                    {
                        TIMESTAMP: float(line_contents[1]),
                        BLE_ADDRESS: line_contents[2],
                        RSSI: int(line_contents[4]),
                    },
                )
            elif data_type in ["ACCE", "GYRO", "MAGN", "BARO"]:
                record = {
                    TIMESTAMP: float(line_contents[1]),
                    ACC_X if data_type == "ACCE" else GYRO_X: float(line_contents[3]),
                    ACC_Y if data_type == "ACCE" else GYRO_Y: float(line_contents[4]),
                    ACC_Z if data_type == "ACCE" else GYRO_Z: float(line_contents[5]),
                }
                if data_type == "BARO":
                    record[PRESSURE] = float(line_contents[3])
                data[data_type].append(record)  # type: ignore[attr-defined]
            elif data_type == "POS3":
                data["POS3"].append(
                    {
                        TIMESTAMP: float(line_contents[1]),
                        POS_X: float(line_contents[3]),
                        POS_Y: float(line_contents[4]),
                        POS_Z: float(line_contents[5]),
                        QUATERNION_0: float(line_contents[6]),
                        QUATERNION_1: float(line_contents[7]),
                        QUATERNION_2: float(line_contents[8]),
                        QUATERNION_3: float(line_contents[9]),
                        FLOOR_NAME: line_contents[10],
                    },
                )

    # Convert lists of dictionaries to DataFrames
    return {key: pd.DataFrame(value) for key, value in data.items()}


def load_sensor_data_from_log(
    log_file_path: str,
) -> tuple[
    pd.DataFrame,
    pd.DataFrame,
    pd.DataFrame,
    pd.DataFrame,
    pd.DataFrame,
    pd.DataFrame,
]:
    """Load sensor data from a log file.

    Args:
    ----
        log_file_path (str): Path to the log file.

    Returns:
    -------
        tuple: Tuple containing DataFrames for accelerometer, gyroscope, magnetometer,
               barometer, ground truth, and BLE scan data.

    """
    data = read_log_data(log_file_path)

    acc_df = data.get("ACCE", pd.DataFrame())
    gyro_df = data.get("GYRO", pd.DataFrame())
    mag_df = data.get("MAGN", pd.DataFrame())
    baro_df = data.get("BARO", pd.DataFrame())  # 気圧データを追加
    gt_df = data.get("POS3", pd.DataFrame())
    ble_df = data.get("BLUE", pd.DataFrame())

    return acc_df, gyro_df, mag_df, baro_df, gt_df, ble_df



import pandas as pd

from src.palkia.const.column_name import TIMESTAMP

from .orientation_estimator import OrientationEstimator
from .step_estimator import StepEstimator
from .trajectory_calculator import TrajectoryCalculator


class PDREstimator:
    def __init__(
        self,
        step_estimator: StepEstimator,
        orientation_estimator: OrientationEstimator,
        trajectory_calculator: TrajectoryCalculator,
    ) -> None:
        self.step_estimator = step_estimator
        self.orientation_estimator = orientation_estimator
        self.trajectory_calculator = trajectory_calculator

    def estimate_trajectory(
        self,
        acc_data: pd.DataFrame,
        gyro_data: pd.DataFrame,
    ) -> pd.DataFrame:
        step_times = self.step_estimator.detect_step_times(acc_data)
        step_orientations = self.orientation_estimator.estimate_step_orientations(
            gyro_data,
            pd.Series(step_times),
        )
        steps_lengths = self.step_estimator.estimate_steps(acc_data, step_orientations)
        # step_orientationsとsteps_lengthsを結合
        steps_data = step_orientations.merge(steps_lengths, on=TIMESTAMP)

        return self.trajectory_calculator.calculate_trajectory(steps_data)



from .orientation_estimator import OrientationEstimator
from .pdr_estimator import PDREstimator
from .step_estimator import StepEstimator
from .three_dimensional_estimator import ThreeDimensionalEstimator
from .trajectory_calculator import TrajectoryCalculator

__all__ = [
    "StepEstimator",
    "OrientationEstimator",
    "TrajectoryCalculator",
    "PDREstimator",
    "ThreeDimensionalEstimator",
]
